#!/usr/bin/lua

wbgen2_version="0.3"

wbgen2_libdir="/home/slayer/wbgen2_svn/wbgen2"

device_family="altera_cyclone3";

options = {};
options.reset_type = "asynchronous";
options.target_interconnect = "wb-classic";
options.register_data_output = false;

dofile(wbgen2_libdir.."/wbgen_common.lua");
dofile(wbgen2_libdir.."/cgen_common.lua");
dofile(wbgen2_libdir.."/cgen_vhdl.lua");
dofile(wbgen2_libdir.."/cgen_verilog.lua");
dofile(wbgen2_libdir.."/cgen_c_headers.lua");
dofile(wbgen2_libdir.."/wbgen_regbank.lua");
dofile(wbgen2_libdir.."/wbgen_rams.lua");
dofile(wbgen2_libdir.."/target_wishbone.lua");

function chk_nil(p,s)
	if(p == nil) then
		die(s.." expected.");
	end
	return p;
end


function parse_args(arg)
	local n=1;

	if(arg[1] == nil) then
		print("wbgen2 version "..wbgen2_version);
		print("(c) Tomasz Wlostowski/CERN BE-Co-HT 2010");
		print("");
		print("usage: "..arg[0].." input_file.wb [options]");
		print("");
		print("Options: ");
		print("-target [classic / pipelined]   - chooses between classic Wishbone bus and HT pipelined Wishbone.");
		print("-lang   [vhdl / verilog]        - chooses the HDL language to be generated");
		print("-vo     [file.vhdl / file.v]    - generates VHDL/Verilog code for the slave Wishbone core.");
		print("-co     [file.h]                - generates C header file containing register definitions and access macros");
		print("-consto [constants.v]           - generates Verilog file containing addresses of all registers/rams and writes them to specified file. Useful for writing testbenches.");
		print("");
		os.exit(0);
	end
	
	input_wb_file = arg[1];

	vhdl_gen_reg_constants = false;
	vlog_gen_reg_constants = false;

	n=2;
	while(arg[n] ~= nil) do
		local sw = arg[n];
		
		if(sw == "-vo") then
			options.output_hdl_file = chk_nil(arg[n+1], "HDL output filename expected");
			n=n+2;
		elseif(sw == "-co") then
			options.output_c_header_file = chk_nil(arg[n+1], "C header output filename expected");
			n=n+2;
		elseif(sw == "-consto") then
			options.output_vlog_constants_file = chk_nil(arg[n+1],"Verilog constants filename expected");
			n=n+2;
		else
			n=n+1;
		end
	
	end

end

parse_args(arg);

dofile(input_wb_file);

if(periph == nil) then die ("missing peripheral declaration"); end
    
foreach_reg(
    function(reg) 
	reg.total_size=0; 
	reg.current_offset=0;
    end
);

foreach_field( fix_prefix );
foreach_field( fix_access );

foreach_reg( fix_prefix );

periph = fix_prefix(periph);

wbgen_count_subblocks();

foreach_field(calc_size);
foreach_reg(check_max_size);
foreach_field(calc_field_offsets);

foreach_reg(calc_address_sizes);

assign_addresses();

tree=gen_bus_logic_wishbone();

cgen_build_signals_ports();

if(options.output_hdl_file ~= nil) then
	cgen_generate_init(options.output_hdl_file)
	cgen_generate_vhdl_code(tree);
	cgen_generate_done();
end

if(options.output_c_header_file ~= nil) then
	cgen_generate_init(options.output_c_header_file)
	cgen_generate_c_header_code();
	cgen_generate_done();
end

if(options.output_vlog_constants_file ~= nil) then
	cgen_gen_vlog_constants(options.output_vlog_constants_file);
end


